// user interface specific routines for USB+Ethernet/DMX Generator  **********
//
// Simple DMX512(incl. Art-Net) controller for STM32F042F6 (20pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// supports the essential features of Art-Net, the MiniDMX protocol
// plus simple MIDI message and ASCII text based command sets
//
// Author and (c)2017-18 Wolfgang Schemmert. The copyright claim and GPL-3 Licence
// is restricted to this file and the depending files
// "main.c", "main.h", "usb-fs.c", "dmx.c" and "ether.c".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org
//
// Contact: www.midi-and-more.de
// A detailled hardware construction and operation manual
// is provided at this website
//
// Note: at present state of art, only the DEBUG object code is working.
// The RELEASE version compiled with EmBitz1.11 does not work. Reason unkown.
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f0xx.h"
#include "globaldefines.h"

// ************************************

extern struct PARAM{
uint32_t VidPid;
uint32_t MacHi;
uint32_t MacLo;
uint32_t LocalIP;
uint32_t Subnet;
uint32_t Gateway;
uint32_t ArtnetIP;
uint32_t ArtSubnet;
uint32_t ArtUniverse;
uint32_t Channel;
} gComParam;  //10 words

extern uint16_t gVidShadow, gPidShadow;

extern void popFlashPage17(void);
extern void pushFlashPage17(void);
void popPresetPage(uint32_t);
void pushPresetPage(uint32_t);
extern void servSystem(void);
extern void msDelay(uint32_t);

//RS-232
#ifdef DMX_DEBUG
extern void sendB2(uint8_t);
extern void sendX8(uint8_t);
extern void sendX32(uint32_t);
#endif
extern void sendUsbX8(uint8_t);
extern void sendUsbN8(uint8_t);
extern void sendUsbX16(uint16_t);
extern void sendUsbX32(uint32_t);
extern void sendUsbS32(uint32_t);
extern void sendUsbString(const uint8_t *);

//USB
extern uint8_t guDeviceClass;
extern uint16_t pullUsbRxBuf(void);
extern void sendUsbB(uint8_t);

//ether.c
extern uint8_t geArtnetOn;  //1=default: Artnet data are send to DMX. 0=block

//dmx.c
extern uint16_t gdDmxAddr, gdDmxMax;
extern uint8_t gdDMXBUF[DMX_BUFLEN+7];          //actual DMX transmit buffer
extern uint8_t gdFINALMEM[DMX_BUFLEN+7];        //final level of fade process
extern uint16_t gdFadeTime;
extern void setDmxAddress(void);
extern void setDmxLevel(uint8_t);
extern void setNextDmxLevel(uint8_t);
extern void setEqual(uint8_t);
extern void msgDmxState(void);
extern void readDmxBuf(void);
extern void sendAddrLevel(void);
extern void sendLevel(uint16_t);

// ************************************

uint32_t gcEtherIP;
uint8_t gcPushByte, gcIgnoreVelo0, gcSetupMode;

const uint8_t gcSetStr[] = {"set"};
const uint8_t gcVidPidStr[] = {" USB Vid/Pid of this device (hex)"};
const uint8_t gcMacIdStr[] = {" Ethernet MAC ID (hex)m5.m4.m3.m2.m1.m0)"};
const uint8_t gcGatewayStr[] = {" Gateway IP (ip3.ip2.ip1.ip0) "};
const uint8_t gcLocalIpStr[] = {" IP of this device (ip3.ip2.ip1.ip0)"};
const uint8_t gcSubnetStr[] = {" Subnet Mask of this device (m3.m2.m1.m0)"};
const uint8_t gcArtIpStr[] = {" Art-Net 'IP Addr A' (0, 2, 10, default=0)"};
const uint8_t gcArtSubnetStr[] = {" Art-Net Subnet (0-15)"};
const uint8_t gcArtUnivStr[] = {" Art-Net Universe DMX OUT (0-15)"};
const uint8_t gcChannelStr[] = {" MIDI base channel (1-16)"};
const uint8_t gcSetupStr[] = {"SETUP>"};

void  ASCII_Interpreter(uint8_t);
void MIDI_Interpreter(uint8_t,uint8_t,uint8_t,uint8_t);
void  Setup_Interpreter(uint8_t);
uint32_t parseHexByte(void);
uint32_t parseHexNibble(void);
uint32_t parseDeciNum(uint32_t);
uint8_t parseDeciNibble(void);
void sendS3(uint32_t);
void sendS2(uint32_t);
uint8_t setVidPid(void);
uint8_t setMacID(void);
uint8_t setEtherIP(void);
void setMidiChannel(void);
void copyComParams(void);
void listCommands(void);
void listSetup(void);


void ASCII_Interpreter(uint8_t CmdCode)  // *********************************
// **************************************************************************
{
uint32_t k;
uint16_t recvword;
const uint8_t AgainStr[] = {"all lights DARK?->press again "};

    if ((CmdCode > 0x60) && (CmdCode < 0x7B)) CmdCode -= 0x20; //->upper case

    sendUsbB(CmdCode);

    switch (CmdCode)  {
    case 'S':
        setDmxAddress();
        break;

    case 'V':
        if ((k = parseDeciNum(255)) > 255)  {
            sendUsbB('?');
            gcPushByte = 0;
        }
        else setDmxLevel(k);
        break;

    case 0x2C:      //comma
         if ((k = parseDeciNum(255)) > 255)  {
            sendUsbB('?');
            gcPushByte = 0;
        }
        else setNextDmxLevel(k);
        break;

    case 'Q':
        msgDmxState();
        break;

    case 'R':
        readDmxBuf();
        break;

    case 'T':
        if ((k = parseDeciNum(127)) > 127)  {
            sendUsbB('?');
            gcPushByte = 0;
        }
        else gdFadeTime = k * 5;
        break;

    case '+':
        if (gdDMXBUF[gdDmxAddr] != 0xFF) {
            gdDMXBUF[gdDmxAddr]++ ;
            gdFINALMEM[gdDmxAddr]++ ;
        }
        break;

   case '-':
        if (gdDMXBUF[gdDmxAddr] != 0x00) {
            gdDMXBUF[gdDmxAddr]-- ;
            gdFINALMEM[gdDmxAddr]-- ;
        }
        break;


    case '=':
        if ((k = parseDeciNum(512)) > 512)  {
            sendUsbB('?');
            gcPushByte = 0;
        }
        else setEqual(k);
        break;

    case '*':
        if ((k = parseDeciNum(1)) > 1)  {
            sendUsbB('?');
            gcPushByte = 0;
        }
        else geArtnetOn = (uint8_t)k;    //1 = Artnet ON (default), 0=OFF
        break;

   case 'C':
        if ((k = parseDeciNum(512)) > 512)  {
            sendUsbB('?');
            gcPushByte = 0;
        }
        else if (k < 24)  {                 //shortest allowed DMX cycle
            sendUsbB('?');
            gcPushByte = 0;
        }
        else gdDmxMax = k;
        break;

    case 'D':
            sendUsbB(0x0D);
            sendUsbString(AgainStr);
            do  {
                recvword = pullUsbRxBuf();
            }   while (recvword == 0) ;
            if ((recvword == 0x8044) || (recvword == 0x8064)) { //'d' or 'D'
                for (k=1; k<513; k++)   {
                    gdDMXBUF[k] = 0 ;     
                    gdFINALMEM[k] = 0;    //Bugfix 27July20
		}
            }
            else  {
                sendUsbB('?');
                gcPushByte = 0;
            }
            break;

    case 0x5C:                            //Backslash
        gcSetupMode = 0xFF;
        sendUsbB(0x0D);
        sendUsbString(gcSetupStr);
        break;

    case '@':
        if ((k = parseDeciNum(13)) > 13)  {
            sendUsbB('?');
            gcPushByte = 0;
        }
        else popPresetPage(k);
        break;

   case '~':
        if ((k = parseDeciNum(13)) > 13)  {
            sendUsbB('?');
            gcPushByte = 0;
        }
        else pushPresetPage(k);
        break;

    case 0x0A:
        sendUsbB(0x0A);
        break;
    case 0x0D:
        sendUsbB(0x0D);
        break;
    case 0x20:
        break;      //ignore SPACE
    default:
        sendUsbB('?');
        gcPushByte = 0;
        break;
    }
}


void MIDI_Interpreter(uint8_t index,uint8_t status,uint8_t data1,uint8_t data2)
// **************************************************************************
{
uint8_t channel;
uint16_t k;

#ifdef DMX_DEBUG
    sendX8(index);
    sendX8(status);
    sendX8(data1);
    sendX8(data2);
#endif

    if (index == 0x08)  {
        index = 0x09;
        status |= 0x10;                     //convert NOTE OFF to NOTE ON
        data2 = 0;                          //with zero velocity
    }
    channel = status & 0x0F;
    if (index == 0x09){
        if ((channel < gComParam.Channel) ||
            (channel > (gComParam.Channel + 3)))  return;//Bugfix 27July20
    }
    else if (channel != gComParam.Channel) return;

    switch (index)  {
    case 0x09:
        if ((data2 == 0) && (gcIgnoreVelo0))  break;
        gdDmxAddr = data1 + 128 * (channel - gComParam.Channel);
        if (gdDmxAddr == 0) gdDmxAddr = 512;
        data2 = data2 << 1;                 //adjust level 0 ... 0xFE(254)
        if (data2 == 0xFE) data2 = 0xFF;
        if ((data2 == 2) && (gcIgnoreVelo0)) data2 = 0;
        setDmxLevel(data2);
        break;

    case 0x0B:
        if (data1 == 1) gdFadeTime = data2 * 5;
        else if ((data1 >= 0x50) && (data1 < 0x54))  {
            if ((data1 == 0x50) && (data2 == 0x00)) gdDmxAddr = 512;
            else gdDmxAddr = 128 * (data1 - 0x50) + data2;
        }
        else if (data1 == 0x54)  {
			data2 = data2 <<1;
			setDmxLevel(data2);     //Bugfix 27July20
        }
        else if (data1 == 0x55)  {
			data2 = (data2 <<1) + 1;
			setDmxLevel(data2);     //Bugfix 27July20
        }
        else if ((data1 == 0x56) && (gdDmxAddr < 512))  {
            gdDmxAddr++ ;
            data2 = data2 <<1;
            setDmxLevel(data2);
        }
        else if ((data1 == 0x57) && (gdDmxAddr < 512)) {
            gdDmxAddr++ ;
            data2 = (data2 <<1) + 1;
            setDmxLevel(data2);
        }
        else if ((data1 == 0x24) && (data2 != 0)) {
            sendAddrLevel();
            if (data2 == 1)  break;
            data2--;
            if (data2 > 9) data2 = 9;
            for (k=gdDmxAddr+1;k < (gdDmxAddr+1+data2);k++) {
                if (k < 513) sendLevel(k);
            }
        }
        else if (data1 == 0x40)  {
            if (data2 < 5) break;		//ignore bad input	bugfix 27July20
            else gdDmxMax = (data2+1) << 2;     //in steps of 4 DMX Tx slots
        }
        else if (data1 == 0x48)  setEqual(data2);
        else if ((data1 == 0x60) && (data2 < 14)) popPresetPage(data2);
        else if ((data1 == 0x70) && (data2 < 14)) pushPresetPage(data2);
        else if (data1 == 0x77)  {  //changed with respect to earlier versions !
            data1 = data2;
            goto PROGRAM_CHANGE;
        }
        break;

    case 0x0C:
PROGRAM_CHANGE:
        if ((data1 == 8) && (gdDMXBUF[gdDmxAddr] != 0x00)) {
            gdDMXBUF[gdDmxAddr]-- ;
            gdFINALMEM[gdDmxAddr]-- ;
        }
        else if ((data1 == 9) && (gdDMXBUF[gdDmxAddr] != 0xFF)) {
            gdDMXBUF[gdDmxAddr]++ ;
            gdFINALMEM[gdDmxAddr]++ ;
        }
        else if (data1 == 0x40) geArtnetOn = 0;
        else if (data1 == 0x41) geArtnetOn = 1;
	    else if (data1 == 0x78) gcIgnoreVelo0 = 0;
        else if (data1 == 0x79) gcIgnoreVelo0 = 0xFF;
        else if (data1 == 0x7F) {
            for (k=1;k<513;k++)  {
		gdDMXBUF[k] = 0 ; 
		gdFINALMEM[k] = 0;    //Bugfix 27July20
            }
        }
        break;
    default:
        break;
    }
}


void  Setup_Interpreter(uint8_t opcode)   // ******************************
// ***************************************************************************
{
uint32_t  parameter;
uint16_t recvword;
const uint8_t AgainStr[] = {"Save to Flash?->press again "};


    if (opcode == 0x0D)  {
        sendUsbB(0x0D);
        sendUsbString(gcSetupStr);
        return;
    }

    if  ((opcode == 0x0A) ||(opcode == 0x20))  {
        return;
    }
    if ((opcode > 0x60) && (opcode <0x7B)) opcode -= 0x20;
    sendUsbB(opcode);
    sendUsbB(0x0D);

    switch(opcode)  {

        case 'M':
            sendUsbString(gcSetStr);
            sendUsbString(gcMacIdStr);
			sendUsbB(':');
            sendUsbB(0x20);
            setMacID();
            break;

        case 'G':
            sendUsbString(gcSetStr);
            sendUsbString(gcGatewayStr);
            sendUsbB(':');
			sendUsbB(0x20);
            if (setEtherIP() == 0) {
                sendUsbB('?');
                break;
            }
            gComParam.Gateway = gcEtherIP;
            break;

       case 'S':
            sendUsbString(gcSetStr);
            sendUsbString(gcSubnetStr);
            sendUsbB(':');
			sendUsbB(0x20);
            if (setEtherIP() == 0) {
                sendUsbB('?');
                break;
            }
            gComParam.Subnet = gcEtherIP;
            break;

        case 'I':
            sendUsbString(gcSetStr);
            sendUsbString(gcLocalIpStr);
            sendUsbB(':');
			sendUsbB(0x20);
            if (setEtherIP() == 0) {
                sendUsbB('?');
                break;
            }
            gComParam.LocalIP = gcEtherIP;
            break;

        case 'A':
            sendUsbString(gcSetStr);
            sendUsbString(gcArtIpStr);
            sendUsbB(':');
			sendUsbB(0x20);
            parameter = parseDeciNum(10);
            if ((parameter != 0) && (parameter != 2) && (parameter != 10))  {
                sendUsbB('?');
                break;
            }
            gComParam.ArtnetIP = parameter;
            break;

        case 'N':
            sendUsbString(gcSetStr);
            sendUsbString(gcArtSubnetStr);
            sendUsbB(':');
			sendUsbB(0x20);
            if ((parameter = parseDeciNum(15)) > 15)  {
                sendUsbB('?');
                break;
            }
            gComParam.ArtSubnet = parameter;
            break;

       case 'U':
            sendUsbString(gcSetStr);
            sendUsbString(gcArtUnivStr);
            sendUsbB(':');
			sendUsbB(0x20);
            if ((parameter = parseDeciNum(15)) > 15)  {
                sendUsbB('?');
                break;
            }
            gComParam.ArtUniverse = parameter;
            break;

        case 'C':
            sendUsbString(gcSetStr);
            sendUsbString(gcChannelStr);
            sendUsbB(':');
			sendUsbB(0x20);
			setMidiChannel();
             break;

        case 'V':
            sendUsbString(gcSetStr);
         	sendUsbString(gcVidPidStr);
			sendUsbB(':');
			sendUsbB(0x20);
            if (setVidPid() == 0) sendUsbB('?');
            break;

        case '!':                   //list actual configuration
            listSetup();
			sendUsbB(0x0D);
            break;

        case '?':
            listCommands();
            sendUsbB(0x0D);
            break;

        case 'E':                   //exit Setup Mode
            sendUsbB('E');
            sendUsbB('x');
            sendUsbB('i');
            sendUsbB('t');
            sendUsbB(0x0D);
            gcSetupMode = 0;
            return;

        case '|':
            sendUsbB(0x0D);
            sendUsbString(AgainStr);
            do  {
                recvword = pullUsbRxBuf();
            }  while (recvword == 0) ;
            if (recvword == 0x807C)  {      // '|'
                pushFlashPage17();
                sendUsbB(0x20);
                sendUsbB('o');
                sendUsbB('k');
                break;
            }
        default: {
            sendUsbB('?');
            break;
        }
    }
    if ((gcPushByte == 0x0D) || (gcPushByte == 0x0A) || (gcPushByte == 0x20))
                                                                gcPushByte = 0;
    sendUsbB(0x0D);
    sendUsbString(gcSetupStr);
}

uint32_t parseHexNibble(void)   // ******************************************
// **************************************************************************
{
uint16_t recvword;
uint32_t nibble;

    while ((recvword = pullUsbRxBuf()) == 0) ;
    nibble = (uint32_t)(recvword & 0x00FF);
    if ((nibble == 0x0A)||(nibble == 0x0D)||(nibble == '.')) return 0xFFFFFFFE;
    if  ((nibble >= 0x30) && (nibble <= 0x39))  {
        sendUsbB((uint8_t)nibble);
        return (nibble - 0x30);
    }
    if ((nibble > 0x60) && (nibble < 0x7B))  nibble -= 0x20; //force uppercase
    if ((nibble >= 'A') && (nibble <= 'F')) {
        sendUsbB((uint8_t)nibble);
        return (nibble - 0x37);
    }
    else return 0xFFFFFFFF;

}


uint32_t  parseHexByte(void)   // ********************************************
// ***************************************************************************
{
uint32_t nibble1, nibble2;

        if ((nibble1 = parseHexNibble())  == 0xFFFFFFFE) return 0xFFFFFFFF;
        else if (nibble1 > 0x0000000F) return 0xFFFFFFFF;
        if ((nibble2 = parseHexNibble()) == 0xFFFFFFFE) return nibble1;
        else if (nibble2 > 0x0000000F)  return 0xFFFFFFFF;
        if (parseHexNibble() != 0xFFFFFFFE) return 0xFFFFFFFF;
        return (uint32_t)((nibble1 << 4) | nibble2);
}


uint32_t parseDeciNum(uint32_t limit)   // **********************************
//unsigned, 0 ...max 65535
// **************************************************************************
{
uint32_t decinum;
uint16_t recvword;
uint8_t digit;

    if (limit > 65535) return 0xFFF10000;
    decinum = 0;
    while (decinum <= limit)  {
        while ((recvword = pullUsbRxBuf()) == 0) ;
        digit = (uint8_t)(recvword & 0x00FF);

        if ((digit < 0x30) || (digit > 0x39 )) {
            gcPushByte = digit;     //flexible delimiter, will be evaluated
                                    //as maybe next command code
            return decinum;
        }
        sendUsbB(digit);
        digit -= 0x30;              //ASCII>num
        decinum  = (decinum * 10) + digit;

//avoid necessity of final CRR when max number of places is reached
        if (decinum > limit) {
            gcPushByte = 0;
            return 0xFFFFFFFF;
        }
        if (limit < 10)  {
            gcPushByte = 0;
            return decinum;
        }
        if (decinum >= 10000) {
            gcPushByte = 0;
            return decinum;
        }
        else if ((decinum >= 1000) && (limit < 10000))  {
            gcPushByte = 0;
            return decinum;
        }
        else if ((decinum >= 100) && (limit < 1000))  {
            gcPushByte = 0;
            return decinum;
        }
        else if ((decinum >= 10) &&(limit < 100))  {
            gcPushByte = 0;
            return decinum;
        }
    }
    gcPushByte = 0;
    return decinum;             //exactly == limit
}


uint8_t parseDeciNibble(void)   // ******************************************
// **************************************************************************
{
uint32_t number;

    if ((number = parseDeciNum(15)) > 15) {
        sendUsbB('?');
        return 0xFF;
    }
    return (uint8_t) number;
}


void sendS3(uint32_t number)   // ************************************************
// **************************************************************************
{
    if (number < 100) sendUsbB('0');
    if (number < 10) sendUsbB('0');
    sendUsbS32(number);
}


void sendS2(uint32_t number)   // ************************************************
// **************************************************************************
{
    if (number < 10) sendUsbB(0x20);
    sendUsbS32(number);
}


uint8_t setVidPid(void)  // ****************************************************
// *****************************************************************************
{
uint8_t nibble;
uint16_t Vid, Pid;
const uint8_t VidStr[] = {"Vid="};
const uint8_t PidStr[] = {"Pid="};
    sendUsbString(VidStr);
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + (uint16_t)nibble;                // *16
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + (uint16_t)nibble;
    sendUsbB(' ');

    sendUsbString(PidStr);
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + (uint16_t)nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + (uint16_t)nibble;

//only write to Shadow after all items are entered correctly
    gVidShadow = Vid;
    gPidShadow = Pid;
    return 1;
}


uint8_t setMacID(void)  // *****************************************************
// *****************************************************************************
{
uint32_t  macHi, macLo, temp;

    temp = parseHexByte();
    if (temp > 255) return 0;
    macHi = temp;
    sendUsbB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macHi = (macHi << 8) | temp;
    sendUsbB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = temp;
    sendUsbB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = (macLo << 8) | temp;
    sendUsbB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = (macLo << 8) | temp;
    sendUsbB('.');
    temp = parseHexByte();
    if (temp > 255) return 0;
    macLo = (macLo << 8) | temp;

// replace not before MAC ID completely parsed
    gComParam.MacHi = macHi;
    gComParam.MacLo = macLo;
    return 1;
}


uint8_t setEtherIP(void)  // ************************************************
// **************************************************************************
{
uint32_t  etherIP, temp;

    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = temp;
    sendUsbB('.');
    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = (etherIP << 8) | temp;
    sendUsbB('.');
    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = (etherIP << 8) | temp;
    sendUsbB('.');
    temp = parseDeciNum(255);
    if (temp > 255) return 0;
    etherIP = (etherIP << 8) | temp;

// replace not before ID completely parsed
    gcEtherIP = etherIP;
    return 1;
}


void setMidiChannel(void)  // ***********************************************
//get the common MIDI channel in decimal format 1...16
// **************************************************************************
{
uint32_t number;

    if(((number = parseDeciNum(16)) > 16) || (number == 0)) {
        sendUsbB(' ');
        sendUsbB('1');
        sendUsbB('-');
        sendUsbB('1');
        sendUsbB('6');
        sendUsbB('!');
        sendUsbB('?');
        return;
    }
    gComParam.Channel = number - 1;
}


void listCommands(void)  // *************************************************
// **************************************************************************
{
const uint8_t SetupStr[] = {"List of Setup Commands:"};
const uint8_t ActualStr[] = {"list Actual Setup"};
const uint8_t FlashStr[] = {"save new setup permanently and activate it"};
const uint8_t ExitStr[] = {"E: exit SETUP"};
const uint8_t TypeStr[] = "USB/DMX+Artnet Generator v" ;
const uint8_t CopyStr[] = " (c)2017 W.Schemmert";

    sendUsbString(TypeStr);
    sendUsbB(HARDWARE_VERSION + 0x30);
    sendUsbB('.');
    sendUsbB(REVNUM + 0x30);
    sendUsbString(CopyStr);

    sendUsbB(0x0D);
    sendUsbString(SetupStr);
    sendUsbB(0x0D);

    sendUsbB('!');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(ActualStr);
    sendUsbB(0x0D);

    sendUsbB('V');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(gcSetStr);
    sendUsbString(gcVidPidStr);
    sendUsbB(0x0D);

    sendUsbB('M');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(gcSetStr);
    sendUsbString(gcMacIdStr);
    sendUsbB(0x0D);

    sendUsbB('G');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(gcSetStr);
    sendUsbString(gcGatewayStr);
    sendUsbB(0x0D);

    sendUsbB('I');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(gcSetStr);
    sendUsbString(gcLocalIpStr);
    sendUsbB(0x0D);

    sendUsbB('S');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(gcSetStr);
    sendUsbString(gcSubnetStr);
    sendUsbB(0x0D);

    sendUsbB('A');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(gcSetStr);
    sendUsbString(gcArtIpStr);
    sendUsbB(0x0D);

    sendUsbB('N');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(gcArtSubnetStr);
    sendUsbString(gcSubnetStr);
    sendUsbB(0x0D);

    sendUsbB('U');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(gcArtUnivStr);
    sendUsbString(gcSubnetStr);
    sendUsbB(0x0D);

    sendUsbB('C');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(gcSetStr);
    sendUsbString(gcChannelStr);
    sendUsbB(0x0D);

    sendUsbB('|');
    sendUsbB(':');
    sendUsbB(0x20);
    sendUsbString(FlashStr);
    sendUsbB(0x0D);
    sendUsbString(ExitStr);
    sendUsbB(0x0D);

}


void listSetup(void)  // *****************************************************
// **************************************************************************
{
const uint8_t SetupStr[] = {"List of Actual Setup:"};

    sendUsbString(SetupStr);
    sendUsbB(0x0D);

    sendUsbX16(gVidShadow & 0x0000FFFF);
    sendUsbB('/');
    sendUsbB(0x20);
    sendUsbX16(gPidShadow & 0x0000FFFF);
    sendUsbString(gcVidPidStr);
    sendUsbB(0x0D);

    sendUsbN8((gComParam.MacHi>>8) & 0xFF);
    sendUsbB('.');
    sendUsbN8(gComParam.MacHi & 0xFF);
    sendUsbB('.');
    sendUsbN8((gComParam.MacLo >> 24) & 0xFF);
    sendUsbB('.');
    sendUsbN8((gComParam.MacLo >> 16) & 0xFF);
    sendUsbB('.');
    sendUsbN8((gComParam.MacLo >> 8) & 0xFF);
    sendUsbB('.');
    sendUsbN8(gComParam.MacLo & 0xFF);
    sendUsbString(gcMacIdStr);
    sendUsbB(0x0D);

    sendS3((gComParam.Gateway >> 24) & 0xFF);
    sendUsbB('.');
    sendS3((gComParam.Gateway >> 16) & 0xFF);
    sendUsbB('.');
    sendS3((gComParam.Gateway >> 8) & 0xFF);
    sendUsbB('.');
    sendS3(gComParam.Gateway & 0xFF);
    sendUsbString(gcGatewayStr);
    sendUsbB(0x0D);

    sendS3((gComParam.LocalIP >> 24) & 0xFF);
    sendUsbB('.');
    sendS3((gComParam.LocalIP >> 16) & 0xFF);
    sendUsbB('.');
    sendS3((gComParam.LocalIP >> 8) & 0xFF);
    sendUsbB('.');
    sendS3(gComParam.LocalIP & 0xFF);
    sendUsbString(gcLocalIpStr);
    sendUsbB(0x0D);

    sendS3((gComParam.Subnet >> 24) &0xFF);
    sendUsbB('.');
    sendS3((gComParam.Subnet >> 16) & 0xFF);
    sendUsbB('.');
    sendS3((gComParam.Subnet >> 8) & 0xFF);
    sendUsbB('.');
    sendS3(gComParam.Subnet & 0xFF);
    sendUsbString(gcSubnetStr);
    sendUsbB(0x0D);

    sendS2(gComParam.ArtnetIP & 0x0000000F);
    sendUsbString(gcArtIpStr);
    sendUsbB(0x0D);

    sendS2(gComParam.ArtSubnet & 0x0000000F);
    sendUsbString(gcArtSubnetStr);
    sendUsbB(0x0D);

    sendS2(gComParam.ArtUniverse & 0x0000000F);
    sendUsbString(gcArtUnivStr);
    sendUsbB(0x0D);

    sendS2((gComParam.Channel & 0x0000000F) +1);
    sendUsbString(gcChannelStr);
    sendUsbB(0x0D);
}

// ##########################################################################
